﻿namespace Connected.Services.Middleware
{
	public abstract class FunctionMiddleware<TArgs, TReturnValue> : ServiceMiddleware<TArgs>, IFunctionMiddleware<TArgs, TReturnValue>
	{
		protected TArgs? Arguments { get; private set; }
		public async Task<TReturnValue?> Invoke(TArgs? args, TReturnValue? result)
		{
			Arguments = args;

			await OnValidate();
			return await OnInvoke(result);
		}

		protected virtual async Task<TReturnValue?> OnInvoke(TReturnValue? result)
		{
			await Task.CompletedTask;

			return result;
		}

		protected virtual async Task OnValidate()
		{
			await Task.CompletedTask;
		}
	}
}
